#include <videoutils.h>
#include <qsqldatabase.h>
#include <mythtv/mythcontext.h>
#include <mythtv/mythmediamonitor.h>
#include <mythtv/mythmedia.h>

bool VideoPlayerPopup::onMediaEvent(MythMediaDevice * mediadevice)
{
    if (mediadevice->getMediaType() == MEDIATYPE_DATA)
    {
        emit retry(); 
        return true;
    }
    else
       return MythPopupBox::onMediaEvent(mediadevice);
}

VideoPlayer::VideoPlayer(QSqlDatabase * ldb, MythDialog * lowner, unsigned int idmetadata)
{
      db=ldb;
      owner = lowner;
      curitem = new Metadata();
      curitem->setID(idmetadata);
      curitem->fillDataFromID(db);
      popup = NULL;
};

VideoPlayer::VideoPlayer(QSqlDatabase * ldb, MythDialog * lowner, 
            QString lfilename)
{
      db=ldb;
      owner = lowner;
      curitem = new Metadata();
      curitem->setFilename(lfilename);
      popup = NULL;
};

VideoPlayer::~VideoPlayer()
{
     if (popup) delete popup;
     if (curitem) delete curitem;
     disconnect();
};

void VideoPlayer::PlayNext(void)
{
    if (playing_time.elapsed()>10000 && curitem->ChildID()>0)
    {
         int childid = curitem->ChildID();

         delete curitem;
         curitem = new Metadata();
         curitem->setID(childid);
         curitem->fillDataFromID(db);
         emit playerNextFile();
         Play();
    }else{
          emit playerTerminated();
    }
};

void VideoPlayer::Play()
{
  //play stuff
  QString defaultPath = "";
  if (curitem->ID()>0)
  {
     //Search if valid storage exist
     Storage * storage = new Storage();
     storage->setID(curitem->Storage());
     storage->fillDataFromID(db);  
     if (storage->getMediaType()!=MEDIATYPE_UNKNOWN)
     {
         MediaMonitor * mon = MediaMonitor::getMediaMonitor();
         if (mon)
         {
            QValueList <MythMediaDevice*> medias = mon->getMedias(
                                                 storage->getMediaType());
            QValueList <MythMediaDevice*>::Iterator itr = medias.begin();
            MythMediaDevice * pDev = NULL;
            bool mediafound = false;
            while (!mediafound && itr!= medias.end())
            {
               pDev = * itr;
               if (pDev->getKeyID() == storage->getKeyID())
                 mediafound = true;
               else
                 itr++;
             
             }
             if (!mediafound || pDev==NULL)
             {
                if (popup) delete popup;
                popup = new VideoPlayerPopup (gContext->GetMainWindow(),
                                             "video player popup");
                popup->addLabel(tr("Waiting for media"));
	        popup->addLabel("");
	        popup->addLabel(tr("Please insert the media called %1")
                                 .arg(storage->getVolumeID())
                           );

                QButton * focusButton
                    = popup->addButton(tr("Cancel"),this, SLOT(slotDoCancel()));
                popup->ShowPopup(this,SLOT(slotDoCancel()));
                focusButton->setFocus();
                connect (popup,SIGNAL(retry()),this,SLOT(slotDoRetry()));
                return;
             }else{
               defaultPath = pDev->getMountPath();
             }
         }
     }
     // play file
     QString command = getCommand(defaultPath);
     if (command == "")
         emit playerTerminated();
     else{
         playing_time.start();
         myth_system(command.local8Bit());

         // if 50% of the length of a video is elapsed
         // then the video is wrote down as seen today
         if (curitem->ID() > 0 &&
                 playing_time.elapsed()>(curitem->Length()/2))
         {
              QDate ldate;
              curitem->setLastSeen(ldate);
              curitem->updateDatabase(db);
         }
         PlayNext();
     }
     
   }else{
     //terminated
     emit playerTerminated();
   }
};

void VideoPlayer::slotDoCancel(void)
{
   //terminated
   cancelPopup();
   emit playerTerminated();
};

void VideoPlayer::slotDoRetry(void)
{
   cancelPopup();
   Play();
};

void VideoPlayer::cancelPopup()
{
    if (popup)
    {
       popup->hide();
       delete popup;
       popup = NULL;
    }
}

QString VideoPlayer::getCommand(QString path)
{
    QString filename = curitem->Filename();
    QString ext = curitem->Filename().section('.',-1);

    QString handler = gContext->GetSetting("VideoDefaultPlayer");
    QString special_handler = curitem->PlayCommand();
    
    //
    //  Does this specific metadata have its own
    //  unique player command?
    //
    if(special_handler.length() > 1)
    {
        handler = special_handler;
    }
    
    else
    {
        //
        //  Do we have a specialized player for this
        //  type of file?
        //
        
        QString extension = filename.section(".", -1, -1);

        QString q_string = QString("SELECT playcommand, use_default FROM "
                                   "videotypes WHERE extension = \"%1\" ;")
                                   .arg(extension);

        QSqlQuery a_query(q_string, db);
    
        if(a_query.isActive() && a_query.numRowsAffected() > 0)
        {
            a_query.next();
            if(!a_query.value(1).toBool())
            {
                //
                //  This file type is defined and
                //  it is not set to use default player
                //

                handler = a_query.value(0).toString();                
            }
        }
    }
    

    // See if this is being handled by a plugin..
    if(gContext->GetMainWindow()->HandleMedia(handler, filename))
	return "" ;

    QString arg = QString("\"%1%2\"")
                .arg(path)
                .arg(filename.replace(QRegExp("\""), "\\\"").utf8().data());

    QString command = "";
    
    // If handler contains %d, substitute default player command
    // This would be used to add additional switches to the default without
    // needing to retype the whole default command.  But, if the
    // command and the default command both contain %s, drop the %s from
    // the default since the new command already has it
    //
    // example: default: mplayer -fs %s
    //          custom : %d -ao alsa9:spdif %s
    //          result : mplayer -fs -ao alsa9:spdif %s
    if(handler.contains("%d"))
    {
	QString default_handler = gContext->GetSetting("VideoDefaultPlayer");
	if(handler.contains("%s") && default_handler.contains("%s"))
	{
		default_handler = default_handler.replace(QRegExp("%s"), "");
	}
	command = handler.replace(QRegExp("%d"), default_handler);
    }

    if(handler.contains("%s"))
    {
        command = handler.replace(QRegExp("%s"), arg);
    }
    else
    {
        command = handler + " " + arg;
    }

    return command;
}

TreeGroupByItem::TreeGroupByItem(UIListGenericTree * parent,
                                 VideoFilterCapability* lcap,
                                 QValueList <VideoFilterCapability*> lgroupby
                                 ): UIListGenericTree(parent,lcap->getName())
{
    groupby = lgroupby;
    groupby.append(lcap);

};
TreeGroupByItem::TreeGroupByItem(UIListGenericTree * parent,QString name)
                                 : UIListGenericTree(parent,name)
{
};

TreeFilterCapabilityItem::TreeFilterCapabilityItem
                                (UIListGenericTree * parent,
                                 VideoFilterCapability* lcap)
                            : UIListGenericTree(parent,
                                   lcap->getName() + " :" +
                                   lcap->getText())
{
    capability = lcap;
};

void TreeFilterCapabilityItem::updateText(){
    setText(capability->getName() + " :" + capability->getText());
}

TreeFilterCapabilityValueItem::TreeFilterCapabilityValueItem
                                (UIListGenericTree * parent,
                                 VideoFilterCapability* lcap,
                                 VideoFilterCapabilityValue* lval)
                            : UIListGenericTree(parent,lval->getText())
{
    capability = lcap;
    value = lval;

};

TreeActionItem::TreeActionItem (UIListGenericTree * parent,
                                QString name)
                            : UIListGenericTree(parent,name)
{
     m_action = NULL;
};

void TreeActionItem::setAction(void * laction){
     m_action=laction;
};

void *TreeActionItem::getAction(){
     return m_action;
};

